/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.prot;

import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;

import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.pat.onlprot.standardProtocolModule.ClientProtocollerInterface;
import de.bsvrz.pat.onlprot.standardProtocolModule.ProtocolModule;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Gibt Metainformation und Nutzdaten eines Datensatzes in einer Zeile aus. Daher ergibt sich eine
 * sehr kompakte Ausgabe. Primär für Testzwecke gedacht.
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:40 $ / ($Author: yvonnes $)
 */
public class FlatProtocoller extends ProtocolModule {
    private static final DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    private static final String SEPARATOR = " \t";


    /**
     * Erzeugt eine neue Instaz des FlatProtocollers mit einem LineProtocoller für die Ausgabe
     */
    public FlatProtocoller() {
        setProtocoller(new LineProtocoller());
    }


    public ClientReceiverInterface initProtocol(ArgumentList argumentList, PrintWriter protocolFile, String[] args) {
        setProtocolFileWriter(protocolFile);
        getProtocoller().writeHeader(args);
        return getProtocoller();
    }


    public String getHelp() {
        return "";
    }


    public void update(ResultData[] results) {
    }


    public void closeProtocol() {
        getProtocoller().writeFooter();
        getProtocolFileWriter().flush();
        getProtocolFileWriter().close();
    }


    // Inner Class
    private class LineProtocoller implements ClientProtocollerInterface {

        public void writeHeader(String[] args) {
            getProtocolFileWriter().println("# BEGIN: " + dateFormat.format(new Date(System.currentTimeMillis())));
            getProtocolFileWriter().println("# Date -> ObjectId -> AtgId -> AspId -> Payload");
        }


        public void update(ResultData[] results) {
            for (ResultData data : results) {
                getProtocolFileWriter().print(dateFormat.format(data.getDataTime()) + SEPARATOR);
                getProtocolFileWriter().print(
                        data.getObject().getId() + SEPARATOR + data.getDataDescription().getAttributeGroup().getId() + SEPARATOR
                                + data.getDataDescription().getAspect().getId() + SEPARATOR);
                writeData(data.getData());
                getProtocolFileWriter().println();
            }
        }


        public void writeFooter() {
            getProtocolFileWriter().println("# END: " + dateFormat.format(new Date(System.currentTimeMillis())));
        }


        private void writeData(Data data) {
            if (data.isPlain()) {
                getProtocolFileWriter().print(data.getName() + "=\"" + data.valueToString() + "\"  ");
            } else if (data.isArray()) {
                getProtocolFileWriter().print("[");
                Iterator i = data.iterator();
                while (i.hasNext()) {
                    writeData((Data) i.next());
                }
                getProtocolFileWriter().print("]");
            } else if (data.isList()) {
                getProtocolFileWriter().print("(");
                Iterator i = data.iterator();
                while (i.hasNext()) {
                    writeData((Data) i.next());
                }
                getProtocolFileWriter().print(")");
            } else {
                System.out.println("oops");
            }
        }
    }
}
